/******************************************************************/
/*  Main Stata program for "Leveraging Lotteries for Value-Added  */
/*       Tables 1, 2, 4, and the "A" panels of Tables 3 & A2      */
/*      Also generates "first step" OLS, RF, and FS estimates     */
/*           Originally created January 29 2015 by PDH            */
/*                Last updated April 4 2017 by PDH                */
/******************************************************************/

clear all
cap log close
set more off
set trace off
set matsize 11000
set maxvar 32767
set seed 42
adopath + "Z:\ado\plus\"

/* directories */
global asgndata   "Z:/BPS/BPS assignment data/"
global cleandata  "Z:/BPS/Peter/BPSVA/Data/"
global distdata   "Z:/BPS/Peter/BPSVA/Data/location/distance files/"
global chartdata  "Z:/charters/MA Lotteries/all lotto data/school by year files/"
global programs   "Z:/BPS/Peter/BPSVA/Programs/"
global tables     "Z:/BPS/Peter/BPSVA/Tables/"
global graphs     "Z:/BPS/Peter/BPSVA/Graphs/"

/* timestamps */
local simsdate    "2015 01 19"
local charterdate "2014 08 10"
local rundate     "07052016"

/* all baseline controls */
local all_covs    "bl_female bl_sch_female bl_black bl_sch_black bl_hisp bl_sch_hisp bl_white bl_sch_white bl_asian bl_sch_asian bl_sped bl_sch_sped bl_lep bl_sch_lep bl_foodfred bl_sch_foodfred bl_susp bl_sch_susp bl_truancy bl_sch_truancy bl_m bl_sch_m bl_e bl_sch_e y_*"

/* baseline VAM controls */
local VAM_covs_m1 "y_*"
local VAM_covs_m2 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy y_*"
local VAM_covs_m3 "bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013 bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 y_*"

/* baseline lottery controls */
local QE_covs_t0 "one"
local QE_covs_t1 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3"
local QE_covs_t2 "y_* bl_m bl_m2 bl_m3 bl_e bl_e2 bl_e3 bl_female bl_black bl_hisp bl_white bl_asian bl_sped bl_lep bl_foodfred bl_susp bl_truancy bl_y_2013"

/* student "types" */
local het_covs "bl_hisp bl_black bl_female bl_foodfred bl_sped bl_lep bl_t_terc"

/* school sectors */
local charters "RoxP 4100205 4190305 MATCH 4280305 APR 4390050 4430205 4440205 UP 4490305 4570205 4590305 BosP 4630205 4750505 4810550 4900305"
local pilots   "1459 2360 2440 4160 4285 4331 4600 4661 4680 350077 350565"
local early_entry "350077 350080 4100205 4280305 4430205 4440205 4490305 4570205 4590305 4630205 4750505 4810550 4900305"

/********************************* program toggles ***************************************/
local ms_subs_setup = 0      /* Generate lottery oversubscription rates                  */
local ms_pref_setup = 0      /* Set up lottery preference sample                         */
local ms_exp_setup  = 0      /* Set up lottery sample                                    */
local ms_obs_setup  = 0      /* Set up OLS sample                                        */
local sumstats      = 0      /* School counts and baseline summary statistics (Tab 1-2)  */
local balance       = 0      /* Balance check  (Tab 2)                                   */
local forecast_test = 0      /* Test for forecast bias  (Tab 3)                          */
local overid_test   = 0      /* Overidentification bias test (Tab 3-4)                   */
local fstep_ests    = 0      /* Joint estimation of "first step" OLS/RF/FS coefficients  */
/******************************** sampling options ***************************************/
local min_enroll    = 25     /* minimum class size of included schools; baseline is 25   */
local min_qesample  = 50     /* minimum q-e sample size; baseline is 50                  */
local ref_school    = 350691 /* VAM Reference school; baseline is Hernandez (350691)     */
local vam_on_qe     = 0      /* =1: use q-e sample to estimate VA, else 0                */
local nocharters    = 0      /* =1: exclude charter lotteries from reduced-form test     */
local bs_reps       = 500    /* # of bootstrap replications for reduced-form test        */
local bs_seed       = 42     /* starting seed for bootstrap                              */                             
/********************************* model options *****************************************/
local VAM_t         = "m"    /* test outcome (m = Math, e = ELA)                         */
local fdiff         = ""     /* first difference outcomes: "d" = gains, "" = levels      */
local het_fx        = 0		 /* =1: estimate/test VAM with type-specific coefficients    */
/*****************************************************************************************/

/*****************************************/
/* SECOND-STEP OVIERD TEST SUBROUTINE ****/
/*****************************************/
{
cap program drop rf_delta
program rf_delta, eclass
	syntax , vam_schs(string) n_vam_schs(real) qe_schs(string) n_qe_schs(real) fs_coefs(string) N_fs_coefs(real)

	matrix theta = e(b)'
	matrix Sigma = e(V)
	
	matrix gtop  = theta[`n_vam_schs'+1..`n_vam_schs'+`n_qe_schs',1]
	matrix Gtop1 = J(`n_qe_schs',`n_vam_schs',0)
	matrix Gtop2 = I(`n_qe_schs')
	matrix Gtop3 = J(`n_qe_schs',`n_fs_coefs',0)		
	
	matrix gbot  = J(`n_qe_schs',1,0)
	matrix Gbot1 = J(`n_qe_schs',`n_vam_schs',0)
	matrix Gbot2 = J(`n_qe_schs',`n_qe_schs',0)
	matrix Gbot3 = J(`n_qe_schs',`n_fs_coefs',0)
	
	local k = 1
	local qes "`qe_schs'"
	local vams "`vam_schs'"
	local title1 ""
	local title2 ""
	forval i = 1/`n_qe_schs' {	
		local qe  = word("`qes'",1)
		local title1 "`title1' QE_`qe'"
		local title2 "`title2' VAM_`qe'"
		forval j = 1/`n_vam_schs' {
			local vam = word("`vams'",1)
			local fs "`qe'_`vam'"
			local temp: list fs - fs_coefs
			if "`temp'" == "" {
				matrix gbot[`i',1] = gbot[`i',1] + _b[vam_`vam']*_b[fs_`qe'_`vam']
				cap matrix Gbot1[`i',`j'] = _b[fs_`qe'_`vam']
				matrix Gbot3[`i',`k'] = _b[vam_`vam']
				local ++k
			}
			local vams: list vams - vam
		}
		local vams "`vam_schs'"
		local qes: list qes - qe
	}	
	
	matrix g = [gtop\gbot]
	matrix Gtop  = [Gtop1,Gtop2,Gtop3]
	matrix Gbot  = [Gbot1,Gbot2,Gbot3]
	matrix G = [Gtop\Gbot]
	
	tempname b V
	matrix `b' = g'
	matrix colnames `b' = `title1' `title2'
	matrix `V'  = G*Sigma*G'
	matrix colnames `V' = `title1' `title2'
	matrix rownames `V' = `title1' `title2'
	ereturn post `b' `V'
end	
}
/*****************************************/
/* ONE-STEP OVIERD TEST SUBROUTINE *******/
/*****************************************/
{
cap program drop rf_sandw
program rf_sandw, eclass
	syntax , weights(string) vam_on_qe(real) vam_covs(string) vam_schs(string) vam_ds(string) n_vam_schs(real) qe_covs(string) qe_schs(string) n_qe_schs(real) qe_test(string)
		preserve
		
		drop y_8

		if `vam_on_qe' == 0 {	
			_rmcoll D_* `vam_covs' [aw=`weights'], forcedrop
			local VAM_Xs "`=r(varlist)'"
			reg Y `VAM_Xs' [aw=`weights']
			predict vamres, r
			foreach var of varlist D_* {
				local temp "`var'"
				local temp2: list VAM_Xs - temp
				local temp3: list VAM_Xs - temp2
				if "`temp3'" == "" {
					drop `var'
					local sch = substr("`var'",3,.)
					local vam_schs: list vam_schs -  sch
					local tempD "D_`sch'"
					local vam_ds: list vam_ds - tempD
					local n_vam_schs = `n_vam_schs' - 1					
				}
			}
			matrix accum M_XX = `VAM_Xs' [aw=`weights']
			matrix S = I(`n_vam_schs')
			matrix S = [S,J(`n_vam_schs',`=rowsof(M_XX)-`n_vam_schs'',0)]
			gen VAM = 0
			foreach s in `vam_schs' {
				replace VAM = _b[D_`s'] if D_`s' == 1
			}
		}
	
		foreach var of varlist riskset_* {
			local sch = substr("`var'",9,.)
			local temp "`qe_schs'"
			local temp2: list temp - sch
			local temp3: list temp - temp2
			if "`temp3'" == "" {
				qui drop if riskset_`sch' > 0 & qesamp
				qui drop riskset_`sch' qual_`sch'
			}
		}

		egen riskset = group(riskset_*) if qesamp
		qui tab riskset if qesamp, gen(d_)
	
		sort sasid
		if `vam_on_qe' == 1 {
			_rmcoll D_* `vam_covs' if qesamp [aw=`weights'], forcedrop
			local VAM_Xs "`=r(varlist)'"
			reg Y `VAM_Xs' if qesamp [aw=`weights']
			predict vamres, r
			foreach var of varlist D_* {
				local temp "`var'"
				local temp2: list VAM_Xs - temp
				local temp3: list VAM_Xs - temp2
				if "`temp3'" == "" {
					drop `var'
					local sch = substr("`var'",3,.)
					local vam_schs: list vam_schs -  sch
					local tempD "D_`sch'"
					local vam_ds: list vam_ds - tempD
					local n_vam_schs = `n_vam_schs' - 1					
				}
			}
			matrix accum M_XX = `VAM_Xs' if qesamp [aw=`weights']
			matrix S = I(`n_vam_schs')
			matrix S = [S,J(`n_vam_schs',`=rowsof(M_XX)-`n_vam_schs'',0)]
			gen VAM = 0
			foreach s in `vam_schs' {
				replace VAM = _b[D_`s'] if D_`s' == 1 & qesamp
			}
		}

		/* Joint FC bias + Overid p-val */
		gen W = Y - VAM
		areg W qual_* `qe_covs' [aw=`weights'], absorb(riskset)
	
		predict rfres, r
		matrix b = e(b)

		qui count if qesamp == 1
		matrix M = r(N)
		
		matrix accum ZD = qual_* `qe_covs' d_* D_* if qesamp == 1 [aw=`weights']
		matrix M_ZZ = ZD[1..`=rowsof(ZD)-`n_vam_schs'-1',1..`=rowsof(ZD)-`n_vam_schs'-1']
		matrix M_ZD = ZD[`=rowsof(ZD)-`n_vam_schs''..`=rowsof(ZD)-1',1..`=rowsof(ZD)-`n_vam_schs'-1']'
		matrix M_XX = M_XX
		
		foreach var of varlist rfres qual_* `qe_covs' d_* {
			qui gen z_`var' = `var'
			qui replace z_`var' = 0 if qesamp == 0
		}
		local QE_Zs ""
		foreach z of varlist qual_* `qe_covs' d_*  {
			qui replace z_`z' = `z'*z_rfres
			local QE_Zs "`QE_Zs' z_`z'"
		}
		gen one2 = one
		foreach x of varlist `VAM_Xs' one2 {
			qui replace `x' = `x'*vamres
		}	
		
		mata X = st_data(.,"`VAM_Xs' one2")
		mata M_XX = st_matrix("M_XX")
		mata M_ZD = st_matrix("M_ZD")
		mata S = st_matrix("S")
		mata P = (M_ZD*S*invsym(M_XX)*X')' 
		local i = 1
		foreach z of varlist qual_* `qe_covs' d_*  {
			qui gen temp = .
			mata: st_store(.,"temp",P[.,`i'])
			qui replace z_`z' = z_`z' - temp
			drop temp
			local ++i
		}
		
		matrix accum Q = `QE_Zs' [aw=`weights'], nocons
		matrix V = invsym(M_ZZ)*(Q)*invsym(M_ZZ)
		matrix b = b[1,1..`n_qe_schs']
		matrix V = V[1..`n_qe_schs',1..`n_qe_schs']		
		
		ereturn post b V
		test `qe_test'
		ereturn scalar F = r(chi2)
		restore

end	
}


/*****************************************/
/* GENERATE OVERSUBSCRIPTION RATES********/
/*****************************************/
if `ms_subs_setup'==1 {

	/* Demand: charters */
	use "${chartdata}new_BU_lottofiles `charterdate'.dta"

	keep if year>=2006 & year<=2013
	drop if sasid == .
	
	rename year yearapp
	rename grade gradeapp
	keep if gradeapp <= 6
	replace school = "UP" if school == "UPAcademy"
	replace school = "RoxP" if school == "UncommonSchools"
	replace school = "MATCH" if school == "MATCH_MS"
	replace school = "BosC" if school == "BosCol"
	replace school = "BosP" if school == "BosPrep"
	replace school = "EdB" if school == "EdBrooke"
	replace school = "EdB2" if school == "EdBrooke2"
	replace school = "EdB3" if school == "EdBrooke3"
	replace school = "Exc" if school == "Excel"
	replace school = "Exc3" if school == "Excel3"
	replace school = "GrH" if school == "GroveH"
	replace school = "KIPP" if school == "KIPP_BOS"
	replace school = "RoxP" if school == "RoxPrep"
	gen masscode = 4120530 if school=="APR"
	replace masscode = 4490305 if school=="BosC"
	replace masscode = 4160305 if school=="BosP"
	replace masscode = 4280305 if school=="EdB"
	replace masscode = 4430205 if school=="EdB2"
	replace masscode = 4570205 if school=="EdB3"
	replace masscode = 4100205 if school=="Exc"
	replace masscode = 4590305 if school=="Exc3"
	replace masscode = 4620305 if school=="GrH"
	replace masscode = 4630205 if school=="KIPP"
	replace masscode = 4690505 if school=="MATCH"
	replace masscode = 4840505 if school=="RoxP"
	replace masscode = 4800405 if school=="UP"
	replace masscode = 4370505 if school=="CoaH"
	replace masscode = 4750505 if school=="DCA"
	
	keep yearapp sasid school masscode gradeapp initial_offer 

	duplicates drop
	
	gen apply = 1
	collapse (sum) apply, by(school masscode yearapp gradeapp)
	tempfile charter_sub
	save `charter_sub'

	/* Demand: pilots/publics */
	use "${cleandata}ms_assign06", clear
	foreach y in 07 08 09 10 11 12 13 {
		append using "${cleandata}ms_assign`y'"
	}
	order sasid studentno yearapp monthapp grade
	
	replace sep = 4272 if sep==2070  // undo Curley split
	forval i = 1/10 {
		replace ch`i' = 4272 if ch`i'==2070
	}
	replace sep = 4178 if sep==2130  // undo Edison split
	forval i = 1/10 {
		replace ch`i' = 4178 if ch`i'==2130
	}
	replace sep = 4055 if sep==2100  // undo King split
	forval i = 1/10 {
		replace ch`i' = 4055 if ch`i'==2100
	}
	replace sep = 4242 if sep==2060  // undo Lewis split
	forval i = 1/10 {
		replace ch`i' = 4242 if ch`i'==2060
	}
	replace sep = 4671 if sep==2350  // undo Mildred split
	forval i = 1/10 {
		replace ch`i' = 4671 if ch`i'==2350
	}
	replace sep = 4323 if sep==2050  // undo Umana split
	forval i = 1/10 {
		replace ch`i' = 4323 if ch`i'==2050
	}
	replace sep = 2260 if sep==1260  // undo Dearborn split
	forval i = 1/10 {
		replace ch`i' = 2260 if ch`i'==1260
	}

	/* Get Masscodes */	
	preserve
	insheet using "${cleandata}ms_schoolcode_xwalk.csv", names clear
	cap drop v*
	keep if sch_masscode!=.
	save "${cleandata}ms_schoolcode_xwalk_tomerge", replace
	restore
	rename sep sch
	merge m:1 sch using "${cleandata}ms_schoolcode_xwalk_tomerge"
	keep if _merge==1 | _merge==3
	drop _merge sch_name
	rename sch sep
	rename sch_masscode sep_masscode
	forval i = 1/10 {
		rename ch`i' sch
		merge m:1 sch using "${cleandata}ms_schoolcode_xwalk_tomerge"
		keep if _merge==1 | _merge==3
		drop _merge sch_name
		rename sch ch`i'
		rename sch_masscode ch`i'_masscode
	}

	gen gradeapp = 6
	gen apply = 1
	collapse (sum) apply, by(ch1 ch1_masscode yearapp gradeapp)
	drop if ch1 == .
	rename ch1 school
	rename ch1_masscode masscode
	tostring school, replace
	drop if school=="2450" & yearapp >= 2011 /* people ranking UP by mistake */

	bys yearapp gradeapp: egen totapp = sum(apply)
	gen frac_apply = apply/totapp
	drop totapp
	
	append using `charter_sub'
	tempfile oversub
	save `oversub'

	/* Enrollment counts */
	forval g= 4/6 {
		use "${cleandata}simsmcaswide `simsdate'", clear
		keep if year`g' >= 2007 & year`g' <= 2014 & boston`g' == 1
		replace masscode`g' = 350020 if masscode`g' == 350420 // Undo Curley Split
		replace masscode`g' = 350375 if masscode`g' == 350425 // Undo Edison Split
		replace masscode`g' = 350376 if masscode`g' == 350410 // Undo King Split
		replace masscode`g' = 350377 if masscode`g' == 350455 // Undo Lewis Split
		replace masscode`g' = 350378 if masscode`g' == 350381 // Undo Mildred Split
		replace masscode`g' = 350656 if masscode`g' == 350405 // Undo Umana Split
		replace masscode`g' = 350657 if masscode`g' == 350653 // Undo Umana Split'
		gen yearapp = year`g' -1

		/* drop small schools */
		gen one = 1
		bys masscode6 year6: egen kidcount = total(one)
		bys masscode6: egen min_kidcount   = min(kidcount)
		keep if min_kidcount >= `min_enroll'
	
		keep sasid yearapp masscode`g'
		gen enroll = 1
		collapse (sum) enroll, by(masscode`g' yearapp)
		rename masscode`g' masscode
		gen gradeapp = `g'
		merge 1:1 yearapp gradeapp masscode using `oversub', keep(match)
		drop _merge
		tempfile sub_enroll_`g'
		save `sub_enroll_`g''
	}
	use `sub_enroll_4'
	append using `sub_enroll_5'
	append using `sub_enroll_6'

	gen oversub = apply/enroll
	collapse (mean) oversub frac_apply, by(school masscode)
	merge 1:1 masscode using "${cleandata}ms_fc_sc_schools", keep(master match)
	gen sch_code = masscode
	tostring sch_code, replace
	replace sch_code = sch if _merge==3
	keep sch_code oversub frac_apply
	rename sch_code sch
	merge 1:1 sch using "${cleandata}VAM_sch_xwalk", keep(using match)
	drop _merge 

	order sch_num
	sort sch_num
	
	saveold "${cleandata}ms_demand", replace version(12)	
}
/*****************************************/
/* SET UP LOTTERY PREFERENCE SAMPLE ******/
/*****************************************/
if `ms_pref_setup'==1 {

	/* charters */
	use "${chartdata}new_BU_lottofiles `charterdate'.dta"

	keep if year>=2006 & year<=2013
	
	rename year yearapp
	rename grade gradeapp

	keep if gradeapp == 6
	replace school = "UP" if school == "UPAcademy"
	replace school = "RoxP" if school == "UncommonSchools"
	replace school = "MATCH" if school == "MATCH_MS"
	replace school = "BosC" if school == "BosCol"
	replace school = "BosP" if school == "BosPrep"
	replace school = "EdB" if school == "EdBrooke"
	replace school = "EdB2" if school == "EdBrooke2"
	replace school = "EdB3" if school == "EdBrooke3"
	replace school = "Exc" if school == "Excel"
	replace school = "Exc3" if school == "Excel3"
	replace school = "GrH" if school == "GroveH"
	replace school = "KIPP" if school == "KIPP_BOS"
	replace school = "RoxP" if school == "RoxPrep"
	keep yearapp sasid school gradeapp initial_offer 

	duplicates drop
	drop if sasid==.
	bys school sasid (yearapp): gen first = _n==1
	keep if first == 1
	drop first
	drop if sasid==.

	levelsof school, local(schools)
	foreach s in `schools' {
		gen apply_`s'  = school=="`s'"
		bys sasid yearapp: egen temp = max(apply_`s')
		replace apply_`s' = temp
		drop temp
	}
	keep sasid yearapp gradeapp apply_*
	duplicates drop
	tempfile charters_6
	save `charters_6', replace

	/* pilots/publics */
	use "${cleandata}ms_assign06", clear
	foreach y in 07 08 09 10 11 12 13 {
		append using "${cleandata}ms_assign`y'"
	}
	order sasid studentno yearapp monthapp grade

	/* Clean school codes */	
	replace sep = 4272 if sep==2070  // undo Curley split
	forval i = 1/10 {
		replace ch`i' = 4272 if ch`i'==2070
	}
	replace sep = 4178 if sep==2130  // undo Edison split
	forval i = 1/10 {
		replace ch`i' = 4178 if ch`i'==2130
	}
	replace sep = 4055 if sep==2100  // undo King split
	forval i = 1/10 {
		replace ch`i' = 4055 if ch`i'==2100
	}
	replace sep = 4242 if sep==2060  // undo Lewis split
	forval i = 1/10 {
		replace ch`i' = 4242 if ch`i'==2060
	}
	replace sep = 4671 if sep==2350  // undo Mildred split
	forval i = 1/10 {
		replace ch`i' = 4671 if ch`i'==2350
	}
	replace sep = 4323 if sep==2050  // undo Umana split
	forval i = 1/10 {
		replace ch`i' = 4323 if ch`i'==2050
	}
	replace sep = 2260 if sep==1260  // undo Dearborn split
	forval i = 1/10 {
		replace ch`i' = 2260 if ch`i'==1260
	}

	/* Get Masscodes */	
	preserve
	insheet using "${cleandata}ms_schoolcode_xwalk.csv", names clear
	cap drop v*
	keep if sch_masscode!=.
	save "${cleandata}ms_schoolcode_xwalk_tomerge", replace
	restore
	rename sep sch
	merge m:1 sch using "${cleandata}ms_schoolcode_xwalk_tomerge"
	keep if _merge==1 | _merge==3
	drop _merge sch_name
	rename sch sep
	rename sch_masscode sep_masscode
	forval i = 1/10 {
		rename ch`i' sch
		merge m:1 sch using "${cleandata}ms_schoolcode_xwalk_tomerge"
		keep if _merge==1 | _merge==3
		drop _merge sch_name
		rename sch ch`i'
		rename sch_masscode ch`i'_masscode
	}

	gen gradeapp = 6
	
	local schools ""
	forval i = 1/2 {
		qui levelsof ch`i', local(sch`i')
			local schools: list schools | sch`i'
	}	
	foreach s in `schools' {
		gen apply_`s' = ch1==`s'
	}

	rename sasid sasid_old
	destring sasid_old, replace
	merge m:1 studentno using "${distdata}crosswalkall", keep(master match)
	destring sasid, replace
	replace sasid = sasid_old if sasid==.
	destring sasid, replace
	drop sasid_old
	drop _merge
	drop if sasid==.

	keep yearapp gradeapp sasid apply_*

	/* combine samples */
	merge 1:1 sasid yearapp using `charters_6'
	drop _merge
	foreach var of varlist apply_* {
		replace `var' = 0 if `var' == .
	}
	
	order sasid gradeapp yearapp apply_*
	bys sasid (yearapp): gen first = _n==1
	keep if first==1
	drop first
	compress
	save "${cleandata}ms_prefs_6th", replace	
}
/*****************************************/
/* SET UP LOTTERY SAMPLE *****************/
/*****************************************/
if `ms_exp_setup'==1 {

	/* charters */
	use "${chartdata}new_BU_lottofiles `charterdate'.dta"
	keep if year>=2006 & year<=2013
	drop if disqualified==1 | lateapplicant==1 
	drop if (outofarea==1 | sibling==1) & school!="UPAcademy"
	drop if applyprioritygroup==1 | applyprioritygroup==3 | sasid==.
	rename year yearapp
	rename grade gradeapp

	keep if gradeapp == 6
	replace school = "UP" if school == "UPAcademy"
	replace school = "RoxP" if school == "UncommonSchools"
	replace school = "MATCH" if school == "MATCH_MS"
	replace school = "BosC" if school == "BosCol"
	replace school = "BosP" if school == "BosPrep"
	replace school = "EdB" if school == "EdBrooke"
	replace school = "EdB2" if school == "EdBrooke2"
	replace school = "EdB3" if school == "EdBrooke3"
	replace school = "Exc" if school == "Excel"
	replace school = "Exc3" if school == "Excel3"
	replace school = "GrH" if school == "GroveH"
	replace school = "KIPP" if school == "KIPP_BOS"
	replace school = "RoxP" if school == "RoxPrep"
	keep yearapp sasid school gradeapp initial_offer 
	duplicates drop

	levelsof school, local(schools)
	foreach s in `schools' {
		if "`s'" == "APR" { 
			gen mcode_`s' = 4120530 
		}
		if "`s'" == "BosC" { 
			gen mcode_`s' = 4490305
		}
		if "`s'" == "BosP" { 
			gen mcode_`s' = 4160305
		}
		if "`s'" == "EdB" { 
			gen mcode_`s' = 4280305
		}
		if "`s'" == "EdB2" { 
			gen mcode_`s' = 4430205
		}
		if "`s'" == "EdB3" { 
			gen mcode_`s' = 4570205
		}
		if "`s'" == "Exc" { 
			gen mcode_`s' = 4100205
		}
		if "`s'" == "Exc3" { 
			gen mcode_`s' = 4590305
		}	
		if "`s'" == "GrH" { 
			gen mcode_`s' = 4620305
		}	
		if "`s'" == "KIPP" { 
			gen mcode_`s' = 4630205
		}
		if "`s'" == "MATCH" { 
			gen mcode_`s' = 4690505
		}
		if "`s'" == "RoxP" { 
			gen mcode_`s' = 4840505
		}
		if "`s'" == "UP" { 
			gen mcode_`s' = 4800405 
		}
		gen apply_`s'       = school=="`s'"
		gen qual_`s'        = apply_`s' * initial_offer
	
		egen riskset_`s'    = group(apply_`s' yearapp)
		replace riskset_`s' = 0 if !apply_`s'
	}
	drop school initial_offer apply_*

	tempfile charters_6
	save `charters_6', replace

	/* pilots/publics */
	use "${cleandata}ms_daa_fc_sc_qual", clear
	gen gradeapp = 6

	rename sasid sasid_old
	destring sasid_old, replace
	merge m:1 studentno using "${distdata}crosswalkall", keep(master match)
	destring sasid, replace
	rename sasid sasid_old2
	rename sasid_old sasid
	replace sasid = sasid_old2 if sasid==.
	drop sasid_old2
	drop _merge
	drop if sasid==.

	keep yearapp gradeapp sasid qual_* riskset_* sep sep_masscode ch1 ch2 fc_hurdle sc_hurdle

	foreach var of varlist riskset_* {
		preserve
		local sch1 = substr("`var'",9,.)
		keep if riskset_`sch1'>0 & ch1==`sch1' & ch2!=`sch1'
		foreach var of varlist riskset_* {
			local sch2 = substr("`var'",9,.)
			replace `var' = 0 if "`sch1'" != "`sch2'"
		}
		foreach var of varlist qual_* {
			local sch2 = substr("`var'",6,.)
			replace `var' = 0 if "`sch1'" != "`sch2'"
		}
		tempfile samp_`sch1'
		save `samp_`sch1'', replace
		restore
	}
	use `samp_1450', clear
	foreach var of varlist riskset_* {
		local s = substr("`var'",9,.)
		append using `samp_`s''
	}
	duplicates drop
	foreach var of varlist riskset_* {
		local s = substr("`var'",9,.)
		levelsof sep_masscode if sep == `s', local(mcode)
		if "`mcode'" != "" {
			gen mcode_`s' = `mcode'
		}
		else {
			gen mcode_`s' = .
		}
	}

	drop sep sep_masscode

	/* combine samples */
	append using `charters_6'
	order sasid gradeapp yearapp
	foreach var of varlist qual_* riskset_* {
		replace `var' = 0 if `var' == .
	}
	foreach var of varlist mcode_* {
		qui summ `var'
		replace `var' = r(mean) if `var' == .
	}

	preserve
	keep mcode_*
	duplicates drop
	gen i = 1
	reshape long mcode_, i(i) j(sch) string
	drop i
	drop if mcode_ == .
	rename mcode_ masscode
	save "${cleandata}ms_fc_sc_schools_v2", replace	
	restore
	drop mcode_*

	compress
	save "${cleandata}ms_fc_sc_exper_6th_v2", replace	
}

/*****************************************/
/* SET UP OLS SAMPLE *********************/
/*****************************************/
if `ms_obs_setup'==1 {
	use "${cleandata}simsmcaswide `simsdate'", clear
	keep if year6 >= 2007 & year6 <= 2014 & boston6 == 1

	/* Fix merges and splits */
	forval g = 5/6 {
		replace masscode`g' = 350020 if masscode`g' == 350420 // Undo Curley Split
		replace masscode`g' = 350375 if masscode`g' == 350425 // Undo Edison Split
		replace masscode`g' = 350376 if masscode`g' == 350410 // Undo King Split
		replace masscode`g' = 350377 if masscode`g' == 350455 // Undo Lewis Split
		replace masscode`g' = 350378 if masscode`g' == 350381 // Undo Mildred Split
		replace masscode`g' = 350656 if masscode`g' == 350405 // Undo Umana Split
		replace masscode`g' = 350657 if masscode`g' == 350653 // Undo Umana Split	
	}

	/* drop small schools */
	gen one = 1
	bys masscode6 year6: egen kidcount = total(one)
	bys masscode6: egen min_kidcount   = min(kidcount)
	keep if min_kidcount >= `min_enroll'

	/* baseline covs/sample restriction */
	keep if boston5 == 1 
	
	foreach t in m e {
		gen bl_`t' = c_bos_`t'rawsc5
		bys masscode5 year5: egen bl_sch_`t' = mean(bl_`t')
		forval e = 2/3 {
			gen bl_`t'`e' = bl_`t'^`e'
			bys masscode5 year5: egen bl_sch_`t'`e' = mean(bl_`t'`e')			
		}
	}
	foreach v in female black hisp white asian sped lep foodfred {
		gen bl_`v'  = `v'5
		bys masscode5 year5: egen bl_sch_`v' = mean(`v'5)
	}
	
	gen bl_susp = inschsusp5 + outschsusp5
	replace bl_susp = 0 if bl_susp < 0
	bys masscode5 year5: egen bl_sch_susp = mean(bl_susp)
	gen bl_truancy = daystruant5
	replace bl_truancy = 0 if bl_truancy < 0
	bys masscode5 year5: egen bl_sch_truancy = mean(bl_truancy)	
	qui tab year6, gen(y_)
	
	gen has_bl = 1
	foreach v of varlist `all_covs' {
		replace has_bl = 0 if `v' == .
	}

	gen bl_y_2013 = year5==2013

	keep if has_bl == 1 & c_bos_mrawsc6!=. & c_bos_erawsc6!=.
	keep sasid masscode6 c_bos_mrawsc6 c_bos_erawsc6 year6 masscode5 `VAM_covs_m4'
	gen c_bos_merawsc6 = c_bos_mrawsc6 + c_bos_erawsc6
	
	compress
	save "${cleandata}ms_obs_6th", replace	
}


/*****************************************/
/* SCHOOL COUNTS AND BASELINE SUMSTATS ***/
/*****************************************/
if `sumstats' == 1 {
	use "${cleandata}ms_obs_6th", clear
	gen masscode = masscode6
	merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
	tostring masscode, replace
	replace masscode = sch if _merge==3
	drop _merge sch

	local VAM_schs ""
	local N_VAM_schs = 0
	qui levelsof masscode, local(schools)

	foreach s in `schools' {
		if "`s'" != "`ref_school'" {
			gen D_`s' = (masscode == "`s'")
			local VAM_schs "`VAM_schs' `s'"
			local ++N_VAM_schs
		}
	}

	merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)
	gen qe = (_merge == 3)
	drop _merge

	/* restrict to sufficiently large experiments */
	local QE_schs ""
	gen in_QE = 0
	foreach var of varlist riskset_* {
		local sch = substr("`var'",9,.)
		qui count if riskset_`sch' > 0 & qe == 1
		disp "school `sch': N = `=r(N)' " _continue
		local N = r(N)
		if `N' < `min_qesample' | "`sch'" == "4592" {
			disp "DROP"
			qui replace qe = 0 if riskset_`sch' > 0
			qui replace gradeapp = . if riskset_`sch' > 0
			qui replace yearapp = . if riskset_`sch' > 0
			qui drop riskset_`sch' qual_`sch'
		}
		else {
			disp ""
			local QE_schs "`QE_schs' `sch'"
			replace in_QE = 1 if D_`sch' == 1
		}
	}	
	bys sasid: egen temp = max(qe)
	replace qe = temp
	drop temp
	keep sasid qe in_QE bl_* D_* 
	duplicates drop
	disp "`QE_schs'"

	matrix sumstats = J(100,100,.)
	local row = 1
	local col = 1
	
	local samp1 ""
	local samp2 "if qe"
	local samp3 "if in_QE"
	local samp4 "if qe & in_QE"
	forval j = 1/4 {
		foreach dep in hisp black white asian female foodfred sped lep susp truancy m e {
			qui summ bl_`dep' `samp`j''
				matrix sumstats[`row',`col']   = r(mean)
				matrix sumstats[`row'+2,`col'] = r(N)
			local row = `row' + 2
		}	
		
		local ++col
		local row = 1
	
	}

	local ++col
	local trads : list schools - ref_school
	foreach sch in `charters' {
		cap count if D_`sch' == 1
		if _rc == 0 {
			local trads: list trads - sch
			matrix sumstats[`row',`col']   = 1
			matrix sumstats[`row',`col'+1] = r(N)
			count if D_`sch' == 1 & qe
			matrix sumstats[`row',`col'+2] = r(N)
			if strpos("`QE_schs'","`sch'") !=0 {
				matrix sumstats[`row',`col'+4] = 1		
			}
			if strpos("`early_entry'","`sch'") ==0 {
				matrix sumstats[`row',`col'+3] = 1
			}
			local ++row
		}
	}
	foreach sch in `pilots' {
		cap count if D_`sch' == 1
		if _rc == 0 {
			local trads: list trads - sch
			matrix sumstats[`row',`col']   = 2
			matrix sumstats[`row',`col'+1] = r(N)	
			count if D_`sch' == 1 & qe
			matrix sumstats[`row',`col'+2] = r(N)
			if strpos("`QE_schs'","`sch'") !=0 {
				matrix sumstats[`row',`col'+4] = 1	
			}
			if strpos("`early_entry'","`sch'") ==0 {
				matrix sumstats[`row',`col'+3] = 1
			}
			local ++row
		}
	}
	foreach sch in `trads' {
		cap count if D_`sch' == 1
		if _rc == 0 {
			matrix sumstats[`row',`col']   = 3
			matrix sumstats[`row',`col'+1] = r(N)	
			count if D_`sch' == 1 & qe
			matrix sumstats[`row',`col'+2] = r(N)
			if strpos("`QE_schs'","`sch'") !=0 {
				matrix sumstats[`row',`col'+4] = 1	
			}
			if strpos("`early_entry'","`sch'") ==0 {
				matrix sumstats[`row',`col'+3] = 1
			}
			local ++row
		}
	}

	clear
	svmat sumstats
	br	
}
/*****************************************/
/* BALANCE CHECK *************************/
/*****************************************/
if `balance' == 1 {
	use "${cleandata}ms_obs_6th", clear
	
	gen masscode = masscode6
	merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
	tostring masscode, replace
	replace masscode = sch if _merge==3
	drop _merge sch

	local VAM_schs ""
	local N_VAM_schs = 0
	qui levelsof masscode, local(schools)

	foreach s in `schools' {
		if "`s'" != "`ref_school'" {
			gen D_`s' = (masscode == "`s'")
			local VAM_schs "`VAM_schs' `s'"
			local ++N_VAM_schs
		}
	}
	preserve
	use "${cleandata}ms_fc_sc_exper_6th_v2", clear
	set seed 42
	gen temp = runiform()
	bys sasid (temp): gen temp2 = _n==1
	bys sasid yearapp: egen keep = max(temp2)
	keep if keep
	drop temp temp2 keep
	tempfile exper_uniq
	save `exper_uniq'
	restore	
	merge 1:m sasid using `exper_uniq', keep(master match)
	gen qe = (_merge == 3)
	drop _merge

	/* restrict to sufficiently large experiments */
	local QE_schs ""
	local N_QE_schs = 0
	foreach var of varlist riskset_* {
		local sch = substr("`var'",9,.)
		qui count if riskset_`sch' > 0 & qe == 1
		disp "school `sch': N = `=r(N)' " _continue
		local N = r(N)
		cap desc D_`sch'
		if `N' < `min_qesample' | _rc!=0 {
			disp "DROP"
			qui replace qe = 0 if riskset_`sch' > 0
			qui replace gradeapp = . if riskset_`sch' > 0
			qui replace yearapp = . if riskset_`sch' > 0
			qui drop riskset_`sch' qual_`sch'
		}
		else {
			disp ""
			local QE_schs "`QE_schs' `sch'"
			local ++N_QE_schs
		}
	}	
	disp "`QE_schs'"
	
	preserve
	use "${cleandata}ms_fc_sc_exper_6th_v2", clear
	keep yearapp riskset_* qual_*
	foreach var of varlist riskset_* {
		local sch = substr("`var'",9,.) 
		if strpos("`QE_schs'","`sch'") == 0 {
			drop riskset_`sch'
			drop qual_`sch'
		}
		else {
			bys riskset_`sch': egen min = min(qual_`sch')
			bys riskset_`sch': egen max = max(qual_`sch')
			replace riskset_`sch' = 0 if min == max
			replace riskset_`sch' = riskset_`sch' > 0
			drop min max
		}
	}
	collapse (max) riskset_*, by(yearapp)
	reshape long riskset_, i(yearapp) j(sch) str
	rename riskset_ oversub
	rename yearapp year5
	rename sch masscode
	tempfile oversub
	save `oversub'
	
	use "${cleandata}ms_obs_6th", clear
	gen masscode = masscode6
	merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
	tostring masscode, replace
	replace masscode = sch if _merge==3
	drop _merge sch
	merge m:1 year5 masscode using `oversub', keep(match)
	gen bl_me = (bl_m + bl_e)/2
	collapse (mean) bl_me, by(oversub year5 masscode)
	bys masscode: egen min = min(oversub)
	bys masscode: egen max = max(oversub)
	keep if min != max
	areg bl_me oversub, absorb(masscode) cluster(masscode)
	restore
	
	foreach var of varlist riskset_* qual_* qe {
		qui bys sasid: egen temp = max(`var')
		qui replace `var' = temp
		drop temp
	}
	keep sasid yqe bl_* qual_* riskset_*  
	duplicates drop
	
	local trad_list "`QE_schs'"
	local chart_list ""
	foreach sch in `charters' {
		cap summ qual_`sch'
		if _rc == 0 {
			local trad_list: list trad_list - sch
			local chart_list "`chart_list' `sch'"
		}
	}
	local pilot_list ""
	foreach sch in `pilots' {
		cap summ qual_`sch'
		if _rc == 0 {
			local trad_list: list trad_list - sch
			local pilot_list "`pilot_list' `sch'"
		}
	}
	foreach sect in chart pilot trad {
		local `sect'_qual ""
		local `sect'_rs ""
		foreach sch in ``sect'_list' {
			local `sect'_qual "``sect'_qual' qual_`sch'"
			local `sect'_rs "``sect'_rs' riskset_`sch'"			
		}	
	}
	egen any_qual = rowmax(qual_*)
	egen any_rs = group(riskset_*), m
	egen temp = rowmax(riskset_*)
	replace any_rs = 0 if temp <= 0 | qe == 0
	drop temp
	egen chart_qual = rowmax(`chart_qual')
	egen chart_rs = group(`chart_rs'), m
	egen temp = rowmax(`chart_rs')
	replace chart_rs = 0 if temp <= 0 | qe == 0
	drop temp
	egen pilot_qual = rowmax(`pilot_qual')
	egen pilot_rs = group(`pilot_rs'), m
	egen temp = rowmax(`pilot_rs')
	replace pilot_rs = 0 if temp <= 0 | qe == 0
	drop temp
	egen trad_qual = rowmax(`trad_qual')
	egen trad_rs = group(`trad_rs'), m
	egen temp = rowmax(`trad_rs')
	replace trad_rs = 0 if temp <= 0 | qe == 0
	drop temp

	matrix balance = J(50,50,.)
	local row = 1
	local col = 1
	
	foreach sect in any chart pilot trad {
		local joint_test ""
		foreach dep in hisp black white female foodfred sped lep susp truancy m e {				
			areg bl_`dep' `sect'_qual if qe == 1 & `sect'_rs>0, r absorb(`sect'_rs) 
				matrix balance[`row',`col']   = _b[`sect'_qual]
				matrix balance[`row'+1,`col'] = _se[`sect'_qual]
				matrix balance[`row'+2,`col'] = e(N)
			qui reg bl_`dep' `sect'_qual if qe == 1 & `sect'_rs>0, absorb(`sect'_rs)
				est store `dep'
			local joint_test "`joint_test' [`dep'_mean]`sect'_qual"
			local row = `row' + 2
		}
	
		suest hisp black white female foodfred sped lep susp truancy m e, r
		test `joint_test'
			matrix balance[`row'+1,`col'] = r(p)			

		local ++col
		local row = 1
	}

	clear
	svmat balance
	br	
}
/*****************************************/
/* TEST FOR FORECAST BIAS ****************/
/*****************************************/
if `forecast_test' == 1 {

	matrix fstep_test = J(20,10,.)
	
	/* Loop over VA Models */
	local modelst = 3
	if `het_fx' == 1 | `nocharters' == 1 {
		local modelst = 3
	}
	forval model = `modelst'/3 {
		/* Loop over QE tests */
		forval test = 2/2 { /* controls as in VA Model 3 */
		local QE_covs `QE_covs_t`test''

		use "${cleandata}ms_obs_6th", clear
		gen masscode = masscode6
		merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
		tostring masscode, replace
		replace masscode = sch if _merge==3
		drop _merge sch
		
		if `het_fx' == 1 {
			gen temp = bl_m + bl_e
			xtile bl_m_terc = bl_m, nq(3)
			xtile bl_e_terc = bl_e, nq(3)
			xtile bl_t_terc = temp, nq(3)
			drop temp
		}
		local VAM_schs ""
		local VAM_Ds ""
		local N_VAM_schs = 0
		qui levelsof masscode, local(schools)
		foreach s in `schools' {
			if "`s'" != "`ref_school'" {
				if `het_fx' == 1 {
					egen temp = group(`het_covs')
					qui levelsof temp, local(het_cov_vals)
					foreach v in `het_cov_vals' {
						qui count if masscode == "`s'" & temp==`v'
						if r(N)>0 {
							gen D_`s'_`v' = (masscode == "`s'" & temp==`v')
							local VAM_schs "`VAM_schs' `s'_`v'"	
							local VAM_Ds "`VAM_Ds' D_`s'_`v'"
							local ++N_VAM_schs
						}
					}
					drop temp
				}
				if `het_fx' == 0 {
					gen D_`s' = (masscode == "`s'")
					local VAM_schs "`VAM_schs' `s'"
					local VAM_Ds "`VAM_Ds' D_`s'"
					local ++N_VAM_schs
				}
			}
		}

		foreach me in `VAM_t' {
			if "`fdiff'" == "" {
				gen Y`me' = c_bos_`me'rawsc6
			}
			if "`fdiff'" == "d" {
				gen Y`me' = c_bos_`me'rawsc6 - bl_`me'			
			}
		}	

		merge 1:1 sasid using "${cleandata}ms_prefs_6th", keep(master match)
		
		foreach var of varlist apply_* {					
			replace `var' = 0 if `var' == .
		}
		drop _merge

		gen one = 1
		foreach me in `VAM_t' {
			if `vam_on_qe' == 0 {
				ivreg2 Y`me' D_* `VAM_covs_m`model''	
				predict vamres`me', r
				local VAM_Xs "`=e(inexog1)'"
				matrix accum M_XX = `VAM_Xs'	
				matrix S = I(`N_VAM_schs')
				matrix S = [S,J(`N_VAM_schs',`=rowsof(M_XX)-`N_VAM_schs'',0)]
				gen VAM`me' = 0
				foreach s in `VAM_schs' {
					if _b[D_`s'] == 0 {
						disp in r "Warning: Conditional VAM not fully-supported"
					}
					replace VAM`me' = _b[D_`s'] if D_`s' == 1
				}
			}
		}
		
		merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)
		gen qesamp = (_merge==3)
		drop _merge	

		/* restrict to sufficiently large experiments */
		local QE_schs ""
		local QE_test ""
		local N_QE_schs = 0
		foreach var of varlist riskset_* {
			local sch = substr("`var'",9,.)
			qui count if qesamp==1 & riskset_`sch' > 0
			local count = r(N)
			disp "school `sch': N = `count' " _continue
			cap desc D_`sch'*
			if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592" | (`nocharters' == 1 & strpos("`charters'","`sch'") > 0) {
				disp "DROP"
				qui replace qesamp = 0 if riskset_`sch' > 0
				qui drop riskset_`sch' qual_`sch'
			}
			else {
				gen samp_`sch' = qesamp & riskset_`sch' > 0
				disp ""
				local QE_schs "`QE_schs' `sch'"
				local QE_test "`QE_test' qual_`sch'"
				local ++N_QE_schs
			}
		}
	
		foreach var of varlist riskset_* qual_* qesamp samp_* {
			qui replace `var' = 0 if qesamp == 0
			qui bys sasid: egen temp = max(`var')
			qui replace `var' = temp
			drop temp
		}	
		drop ch1 ch2 fc_hurdle sc_hurdle
		duplicates drop

		foreach sch in `QE_schs' {
			qui tab riskset_`sch', gen(d_`sch'_)
		}

		sort sasid
		if `vam_on_qe' == 1 {
			foreach var of varlist D_* {
				qui count if `var' > 0 & qesamp
				if r(N)==0 {
					drop `var'
					local sch = substr("`var'",3,.)
					local VAM_schs: list VAM_schs -  sch
					local tempD "D_`sch'"
					local VAM_Ds: list VAM_Ds - tempD
					local N_VAM_schs = `N_VAM_schs' - 1				
				}
			}
			foreach me in `VAM_t' {
				ivreg2 Y`me' D_* `VAM_covs_m`model'' if qesamp
				predict vamres`me' if qesamp, r
				local VAM_Xs "`=e(inexog1)'"
				matrix accum M_XX = `VAM_Xs' if qesamp
				matrix S = I(`N_VAM_schs')
				matrix S = [S,J(`N_VAM_schs',`=rowsof(M_XX)-`N_VAM_schs'',0)]
				gen VAM`me' = 0
				foreach s in `VAM_schs' {
					if _b[D_`s'] == 0 {
						disp in r "Warning: Conditional VAM not fully-supported"
					}
					replace VAM`me' = _b[D_`s'] if D_`s' == 1 & qesamp
				}
			}
		}
		qui _rmcoll `QE_covs' d_*_* if qesamp==1, forcedrop
		local QE_Xs "`=r(varlist)'"

		if "`VAM_t'" == "m e" {
			expand 2, gen(copy)
			foreach v in Y VAM vamres {
				gen `v' = `v'm if copy==0
				replace `v' = `v'e if copy==1
			}
			local QE_test ""
			foreach v of varlist qual_* {
				gen `v'_0 = `v'*(1-copy)
				gen `v'_1 = `v'*copy
				drop `v'
				local QE_test "`QE_test' `v'_0 `v'_1"
			}
			local QE_Xs_tmp ""
			foreach v of varlist `QE_Xs' {
				gen `v'_0 = `v'*(1-copy)
				gen `v'_1 = `v'*copy	
				local QE_Xs_tmp "`QE_Xs_tmp' `v'_0 `v'_1"
			}
			local QE_Xs "`QE_Xs_tmp' copy"
		}

		/* FC bias test */
		ivreg2 Y (VAM = qual_*) `QE_Xs' if qesamp==1, gmm2s cluster(sasid) partial(`QE_Xs')
			matrix fstep_test[5,`model'] = e(widstat)

		foreach var of varlist Y VAM qual_* {
			qui reg `var' `QE_Xs' if qesamp==1
			qui predict r_`var', r
		}
		ivregress 2sls r_Y (r_VAM = r_qual_*) if qesamp==1, nocons
		predict ivres, r
		matrix b = e(b)
		
		qui count
		matrix N = r(N)
		qui count if qesamp == 1
		matrix M = r(N)
		
		matrix accum ZD = r_qual_* D_* if qesamp == 1
		matrix M_ZZ = ZD[1..`=rowsof(ZD)-`N_VAM_schs'-1',1..`=rowsof(ZD)-`N_VAM_schs'-1']
		matrix M_ZD = ZD[`=rowsof(ZD)-`N_VAM_schs''..`=rowsof(ZD)-1',1..`=rowsof(ZD)-`N_VAM_schs'-1']'

		local temp = 0
		foreach var of varlist r_VAM {
			local ++temp
		}
		matrix accum EZY = r_Y r_VAM r_qual_* if qesamp == 1
		matrix M_ZY = EZY[`=`temp'+2'..`=rowsof(EZY)-1',1]
		matrix M_EZ = EZY[2..`=`temp'+1',`=`temp'+2'..`=rowsof(EZY)-1']	
		matrix M_XX = M_XX
		if "`VAM_t'" == "m e" {
			matrix M_XX = 2*M_XX
		}

		foreach var of varlist ivres r_qual_* {
			qui gen z_`var' = `var'
			qui replace z_`var' = 0 if qesamp == 0
		}
		local QE_Zs ""
		foreach z of varlist r_qual_* {
			qui replace z_`z' = z_`z'*z_ivres
			local QE_Zs "`QE_Zs' z_`z'"
		}
		gen one2 = one		
		foreach x of varlist `VAM_Xs' one2 {
			qui replace `x' = `x'*vamres
		}	
		
		sort sasid 
		cap sort sasid copy
		
		mata X = st_data(.,"`VAM_Xs' one2")
		mata M_XX = st_matrix("M_XX")
		mata M_ZD = st_matrix("M_ZD")
		mata S = st_matrix("S")
		mata P = (M_ZD*S*invsym(M_XX)*X')' 

		local i = 1
		foreach z of varlist r_qual_*  {
			qui gen temp = .
			mata: st_store(.,"temp",P[.,`i'])
			qui replace z_`z' = z_`z' - temp
			drop temp
			local ++i
		}

		matrix opaccum Q = `QE_Zs', group(sasid) opvar(one) nocons
		matrix V = (invsym(M_EZ*invsym(M_ZZ)*M_EZ')*M_EZ*(invsym(M_ZZ)*(Q)*invsym(M_ZZ))*M_EZ'*invsym(M_EZ*invsym(M_ZZ)*M_EZ'))
		matrix b = b[1,1]
		matrix V = V[1,1]		
		matrix colnames b = VAM
		matrix colnames V = VAM
		matrix rownames V = VAM
		ereturn post b V
		ereturn display
			matrix fstep_test[1,`model'] = _b[VAM]
			matrix fstep_test[2,`model'] = _se[VAM]			
		test VAM = 1
			matrix fstep_test[4,`model'] = r(p)			
		
		forval i = 1/2 {
			matrix beta_GMM = invsym(M_EZ*invsym(Q)*M_EZ')*M_EZ*invsym(Q)*M_ZY
			drop ivres
			gen ivres = r_Y - beta_GMM[1,1]*r_VAM

			drop z_*
			foreach var of varlist ivres r_qual_* {
				qui gen z_`var' = `var'
				qui replace z_`var' = 0 if qesamp==0
			}
			local QE_Zs ""
			foreach z of varlist r_qual_* {
				qui replace z_`z' = z_`z'*z_ivres
				local QE_Zs "`QE_Zs' z_`z'"
			}

			local i = 1
			foreach z of varlist r_qual_*  {
				qui gen temp = .
				mata: st_store(.,"temp",P[.,`i'])
				qui replace z_`z' = z_`z' - temp
				drop temp
				local ++i
			}
			matrix opaccum Q = `QE_Zs', group(sasid) opvar(one) nocons
			matrix V = (invsym(M_EZ*invsym(Q)*M_EZ'))
			matrix b = beta_GMM[1,1]
			matrix V = V[1,1]		
			matrix colnames b = VAM
			matrix colnames V = VAM
			matrix rownames V = VAM
			ereturn post b V
			ereturn display
				matrix fstep_test[11,`model'] = _b[VAM]
				matrix fstep_test[12,`model'] = _se[VAM]			
			test VAM = 1
				matrix fstep_test[13,`model'] = r(p)			
		}
		
		matrix opaccum Q = `QE_Zs' if qesamp==1, group(sasid) opvar(one) nocons
		matrix accum Ze = ivres r_qual_* if qesamp==1, nocons
		matrix Ze = Ze[2...,1]
		matrix J = Ze'*invsym(Q)*Ze
			if "`VAM_t'" == "m e" { 
				matrix fstep_test[6,`model'] = chi2tail(`=2*`N_QE_schs'-1',J[1,1])
			}
			else {
				matrix fstep_test[6,`model'] = chi2tail(`N_QE_schs'-1,J[1,1])			
			}	
		}
	}
	clear
	svmat fstep_test

	if `nocharters' == 0 {
		cap mkdir "${tables}ests`rundate'"
		outsheet using "${tables}ests`rundate'/fcast_bias_test`test'_samp`vam_on_qe'_het`het_fx'.csv", replace
	}
	br
}
/*****************************************/
/* TEST FOR BIAS (OVERIDENTIFICATION) ****/
/*****************************************/
if `overid_test' == 1 {

	matrix fstep_bs_test = J(1,5,.)
	
	/* Get school #s and lists */
	use "${cleandata}ms_obs_6th", clear
	gen masscode = masscode6
	merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
	tostring masscode, replace
	replace masscode = sch if _merge==3
	drop _merge sch

	if `het_fx' == 1 {
		gen temp = bl_m + bl_e
		xtile bl_m_terc = bl_m, nq(3)
		xtile bl_e_terc = bl_e, nq(3)
		xtile bl_t_terc = temp, nq(3)
		drop temp
	}
	local VAM_schs ""
	local VAM_Ds ""
	local N_VAM_schs = 0
	qui levelsof masscode, local(schools)
	foreach s in `schools' {
		if "`s'" != "`ref_school'" {
			if `het_fx' == 1 {
				egen temp = group(`het_covs')
				qui levelsof temp, local(het_cov_vals)
				foreach v in `het_cov_vals' {
					qui count if masscode == "`s'" & temp==`v'
					if r(N)>0 {
						gen D_`s'_`v' = (masscode == "`s'" & temp==`v')
						local VAM_schs "`VAM_schs' `s'_`v'"	
						local VAM_Ds "`VAM_Ds' D_`s'_`v'"
						local ++N_VAM_schs
					}
				}
				drop temp
			}
			if `het_fx' == 0 {
				gen D_`s' = (masscode == "`s'")
				local VAM_schs "`VAM_schs' `s'"
				local VAM_Ds "`VAM_Ds' D_`s'"
				local ++N_VAM_schs
			}
		}
	}
	
	merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)
	gen qesamp = _merge==3
	drop _merge

	local QE_schs ""
	local N_QE_schs = 0
	foreach var of varlist riskset_* {
		local sch = substr("`var'",9,.)
		qui count if qesamp & riskset_`sch' > 0
		local count = r(N)
		disp "school `sch': N = `count' " _continue
		cap desc D_`sch'*
		if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592" | (`nocharters' == 1 & strpos("`charters'","`sch'") > 0) {
			disp "DROP"
			qui replace qesamp = 0 if riskset_`sch' > 0
			qui drop riskset_`sch' qual_`sch'
		}
		else {
			gen samp_`sch' = qesamp & riskset_`sch' > 0
			disp ""
			local QE_schs "`QE_schs' `sch'"
			local ++N_QE_schs
		}
	}
		
		
	forval model = 2/2 {
		/* Loop over QE tests */
		forval test = 2/2 { /* controls as in VA Model 3 */

		use "${cleandata}ms_obs_6th", clear
		gen masscode = masscode6			
			
		merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
		tostring masscode, replace
		replace masscode = sch if _merge==3
		drop _merge sch
		
		if `het_fx' == 1 {
			gen temp = bl_m + bl_e
			xtile bl_m_terc = bl_m, nq(3)
			xtile bl_e_terc = bl_e, nq(3)
			xtile bl_t_terc = temp, nq(3)
			drop temp
		}
	
		qui levelsof masscode, local(schools)
		foreach s in `schools' {
			if "`s'" != "`ref_school'" {
			if `het_fx' == 1 {
				egen temp = group(`het_covs')
				qui levelsof temp, local(het_cov_vals)
				foreach v in `het_cov_vals' {
					qui count if masscode == "`s'" & temp==`v'
					if r(N)>0 {
						gen D_`s'_`v' = (masscode == "`s'" & temp==`v')
					}
				}
				drop temp
			}
			if `het_fx' == 0 {
				gen D_`s' = (masscode == "`s'")
			}
			}
		}

		foreach me in `VAM_t' {
			if "`fdiff'" == "" {
				gen Y`me' = c_bos_`me'rawsc6
			}
			if "`fdiff'" == "d" {
				gen Y`me' = c_bos_`me'rawsc6 - bl_`me'			
			}
		}	
		
		merge 1:1 sasid using "${cleandata}ms_prefs_6th", keep(master match)
		
		foreach var of varlist apply_* {					
			replace `var' = 0 if `var' == .
		}
		drop _merge
			
		merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(master match)		
		drop _merge	
		gen qesamp = 0
		foreach sch in `QE_schs' {
			replace qesamp = 1 if riskset_`sch' > 0 & riskset_`sch' ! = .
		}
		
		gen one = 1
		foreach var of varlist riskset_* qual_* qesamp {
			qui replace `var' = 0 if qesamp == 0
			qui bys sasid: egen temp = max(`var')
			qui replace `var' = temp
			drop temp
		}	
		drop ch1 ch2 fc_hurdle sc_hurdle
		duplicates drop
		
		/* Bootstrap F stat */
		local QE_test ""
		foreach sch in `QE_schs' {
			if "`VAM_t'" != "m e" {
				local QE_test "`QE_test' (qual_`sch' = 0)"
			}
			if "`VAM_t'" == "m e" {
				local QE_test "`QE_test' (qual_`sch'_0 = 0) (qual_`sch'_1 = 0)"
			}			
		}	
		
		rf_sandw, weights(one) vam_on_qe(`vam_on_qe') vam_covs("`VAM_covs_m`model''") vam_schs("`VAM_schs'") vam_ds("`VAM_Ds'") n_vam_schs(`N_VAM_schs') qe_covs("`QE_covs_t`test''") qe_schs("`QE_schs'") n_qe_schs(`N_QE_schs') qe_test("`QE_test'")
		local Fest = e(F)

		local QE_test ""
		foreach sch in `QE_schs' {
			if "`VAM_t'" != "m e" {
				local QE_test "`QE_test' (qual_`sch' = `=_b[qual_`sch']')"
			}
			if "`VAM_t'" == "m e" {
				local QE_test "`QE_test' (qual_`sch'_0 = `=_b[qual_`sch'_0]') (qual_`sch'_1 = `=_b[qual_`sch'_1]')"
			}			
		}

		set seed `bs_seed'
		matrix bs_F = J(`bs_reps',1,.)
		forval b = 1/`bs_reps' {
			disp "iter `b': " _continue
			qui {
			gen bs_wgt = rgamma(1,1)
			rf_sandw, weights("bs_wgt") vam_on_qe(`vam_on_qe') vam_covs("`VAM_covs_m`model''") vam_schs("`VAM_schs'") vam_ds("`VAM_Ds'") n_vam_schs(`N_VAM_schs') qe_covs("`QE_covs_t`test''") qe_schs("`QE_schs'") n_qe_schs(`N_QE_schs') qe_test("`QE_test'")
			drop bs_wgt
			matrix bs_F[`b',1] = e(F)
			}			
			disp "`=round(`=e(F)',0.01)'"
		}
		clear
		svmat bs_F
		count if bs_F1 > `Fest'
	
		matrix fstep_bs_test[1,`model'] = r(N)/`bs_reps'	
		}

	}
	clear
	svmat fstep_bs_test
	br

}
/*****************************************/
/* JOINT ESTIMATION OF FIRST STEP PARAMS */
/*****************************************/
if `fstep_ests' == 1 {

	/* Loop over VAM models */
	forval model = 2/2 {
		/* Loop over QE tests */
		forval test = 2/2 { /* controls as in VA Model 3 */
		local QE_covs `QE_covs_t`test''

		use "${cleandata}ms_obs_6th", clear
		gen masscode = masscode6
		merge m:1 masscode using "${cleandata}ms_fc_sc_schools_v2", keep(master match)
		tostring masscode, replace
		replace masscode = sch if _merge==3
		drop _merge sch
		
		local VAM_schs ""
		local N_VAM_schs = 0
		qui levelsof masscode, local(schools)
		foreach s in `schools' {
			if "`s'" != "`ref_school'" {
				gen D_`s' = (masscode == "`s'")
				local VAM_schs "`VAM_schs' `s'"
				local ++N_VAM_schs
			}
		}

		preserve
		clear
		set obs `=`N_VAM_schs'+1'
		gen sch = "`ref_school'" if _n==1
		gen sch_num = 0 if _n==1
		forval j = 1/`N_VAM_schs' {
			replace sch = word("`VAM_schs'",`j') if _n==`j'+1
			replace sch_num = `j' if _n==`j'+1
		}
		save "${cleandata}VAM_sch_xwalk", replace
		restore

		merge 1:1 sasid using "${cleandata}ms_prefs_6th", keep(master match)

		foreach var of varlist apply_* {					
			replace `var' = 0 if `var' == .
		}
		drop _merge
	
		if `vam_on_qe' == 0 {
			preserve
				gen samp = "VAM"
				if "`fdiff'" == "" {
					gen Y = c_bos_`VAM_t'rawsc6
				}
				if "`fdiff'" == "d" {
					gen Y = c_bos_`VAM_t'rawsc6 - bl_`VAM_t'				
				}
				keep sasid samp Y D_* `VAM_covs_m`model'' masscode year6
				tempfile samp_VAM
				save `samp_VAM', replace
			restore
		}
		
		merge 1:m sasid using "${cleandata}ms_fc_sc_exper_6th_v2", keep(match)
		drop _merge

		/* restrict to sufficiently large experiments */
		local QE_schs ""
		local N_QE_schs = 0
		foreach var of varlist riskset_* {
			local sch = substr("`var'",9,.)
			qui count if riskset_`sch' > 0
			local count = r(N)
			disp "school `sch': N = `count' " _continue
			cap desc D_`sch'
			if `count' < `min_qesample' | _rc != 0  | "`sch'" == "4592" {
				disp "DROP"
				qui drop if riskset_`sch' > 0
				qui drop riskset_`sch' qual_`sch'
			}
			else {
				disp ""
				local QE_schs "`QE_schs' `sch'"
				local ++N_QE_schs
			}
		}

		if `vam_on_qe' == 1 {
			preserve
				gen samp = "VAM"
				if "`fdiff'" == "" {
					gen Y = c_bos_`VAM_t'rawsc6
				}
				if "`fdiff'" == "d" {
					gen Y = c_bos_`VAM_t'rawsc6 - bl_`VAM_t'				
				}
				keep sasid samp Y D_* `VAM_covs_m`model''
				tempfile samp_VAM
				save `samp_VAM', replace
			restore
		}

		/* set up test data */
		local copies = `N_VAM_schs' + 1
		local i = 1
		gen one = 1

		local QE_stk_cont ""
		foreach qe in `QE_schs' {
			preserve
			keep if riskset_`qe' > 0
			expand `copies'
			bys sasid: gen copy = _n - 1
			gen samp = "QE_`qe'_RF" if copy == 0
			if "`fdiff'" == "" {
				gen Y = c_bos_`VAM_t'rawsc6 if copy == 0
			}
			if "`fdiff'" == "d" {
				gen Y = c_bos_`VAM_t'rawsc6 - bl_`VAM_t' if copy == 0				
			}
			gen RF_`qe' = qual_`qe'*(copy==0)
			foreach cont of varlist `QE_covs' {
				gen c_`cont'_RF = `cont'*(copy == 0)
			}
			local j = 1
			foreach vam in `VAM_schs' {
				replace samp = "QE_`qe'_" + "`vam'" if copy == `j'
				replace Y = D_`vam' if copy == `j'
				gen FS_`qe'_`vam' = qual_`qe'*(copy == `j')
					foreach cont of varlist `QE_covs' {
						gen c_`cont'_FS`j' = `cont'*(copy == `j')
					}
				local ++j
			}
			egen riskset_for_`qe' = group(riskset_`qe' copy)
			keep sasid samp Y RF_`qe' FS_`qe'_* c_*_RF c_*_FS* riskset_for_`qe' masscode year6
			*qui compress
			tempfile samp_`qe'
			save `samp_`qe'', replace
			restore
		}

		use `samp_VAM', clear
		foreach qe in `QE_schs' {
			append using `samp_`qe''
		}
		foreach var of varlist D_* `VAM_covs_m`model'' RF_* FS_* c_*_RF c_*_FS* riskset_for_* {
			replace `var' = 0 if `var' == .
		}
		egen riskset = group(riskset_for_*)
		
		/* test regression    */
		drop y_1
		egen clust = group(masscode year6)
		areg Y D_* RF_* FS_*_* `VAM_covs_m`model'' c_*_RF c_*_FS*, absorb(riskset) cluster(sasid)
			matrix ests = e(b)
			matrix vce  = e(V)
		cap mkdir "${tables}ests`rundate'"
	
		clear
		xsvmat vce, rowname(entry) fast
		local max = _N
		local entries ""
		forval i = 1/`max' {
			qui levelsof entry if _n==`i', local(temp)
			local temp `temp'
			if "`temp'" == "_cons" | substr("`temp'",1,2) == "o." {
				local temp "temp`i'"
			}
			local entries "`entries' `temp'"
			rename vce`i' `temp'
		}
		cap drop bl_*
		cap drop y_* 
		cap drop temp*
		cap drop c_* 
		drop if substr(entry,1,3) == "bl_" | substr(entry,1,2) == "y_" | substr(entry,1,2) == "c_" | entry == "_cons"
		drop entry

		local j = 1

		foreach vam in `VAM_schs' {		
			rename D_`vam' VAM_`j'
			local ++j
		}

		local i = 1
		foreach sch1 in `VAM_schs' {
			cap rename RF_`sch1' RF_`i'
			local j = 1 
			foreach sch2 in `VAM_schs' {
				cap rename FS_`sch1'_`sch2' FS_`i'_`j'
				local ++j
			}
			local ++i
		}

		format * %20.0g
		egen tempM = rowmax(*)
		egen tempm = rowmin(*)
		drop if tempM == 0 & tempm == 0
		drop tempM tempm
		outsheet using "${tables}ests`rundate'/VAM_vce_m`model'`fdiff'_test`test'_samp`vam_on_qe'.csv", comma replace
		
		clear
		svmat ests
		forval i = 1/`max' {
			local temp = word("`entries'",1)
			rename ests`i' `temp'
			local entries: list entries - temp
		}	
		cap drop bl_*
		cap drop y_* 
		cap drop temp*
		cap drop c_* 
		set obs 3
		local i = 1
		foreach sch1 in `VAM_schs' {	
			if strpos("`charters'","`sch1'")!=0 {
				replace D_`sch1' = 1 in 2
			}
			if strpos("`pilots'","`sch1'")!=0 {
				replace D_`sch1' = 1 in 3
			}
			rename D_`sch1' VAM_`i'
			cap rename RF_`sch1' RF_`i'
			local j = 1 
			foreach sch2 in `VAM_schs' {
				cap rename FS_`sch1'_`sch2' FS_`i'_`j'
				local ++j
			}
			local ++i
		}
		format * %20.0g
		outsheet using "${tables}ests`rundate'/VAM_ests_m`model'`fdiff'_test`test'_samp`vam_on_qe'.csv" in 1, comma replace
		keep VAM_*
		drop if _n==1
		gen sect = _n
		reshape long VAM_, i(sect) j(sch)
		reshape wide VAM_, i(sch) j(sect)
		rename VAM_1 charter
		rename VAM_2 pilot
		replace charter = 0 if charter == .
		replace pilot = 0 if pilot == .
		outsheet using "${tables}ests`rundate'/VAM_sects.csv", comma replace	
		}
	}
}
